@@ -13,7 +13,7 @@
 #include <kmainwindow.h>
 
 #include <QJSEngine>
-#include <QtTestWidgets>
+#include <QTest>
 
 #include "testutils.h"
 
@@ -67,11 +67,10 @@ void BugTest::tryCrash()
     QFile sourceFile(QLatin1String(TEST_DATA_DIR "bug317111.js"));
     QVERIFY(sourceFile.open(QFile::ReadOnly));
     QTextStream stream(&sourceFile);
-    stream.setCodec("UTF8");
     QString code = stream.readAll();
     sourceFile.close();
     // execute script
-    result = env->engine()->evaluate(code, QLatin1String(TEST_DATA_DIR "bug317111.txt"), 1);
+    result = env->engine()->evaluate(code, QStringLiteral(TEST_DATA_DIR "bug317111.txt"), 1);
     QVERIFY2(!result.isError(), result.toString().toUtf8().constData());
 
     qDebug() << "PASS (no crash)";
