@@ -12,12 +12,14 @@
 #include <kateglobal.h>
 #include <kateview.h>
 #include <ktexteditor/inlinenote.h>
-#include <ktexteditor/inlinenoteinterface.h>
 #include <ktexteditor/inlinenoteprovider.h>
 
 #include <QPainter>
 #include <QTemporaryFile>
-#include <QtTestWidgets>
+#include <QTest>
+#include <QTestMouseEvent>
+
+#include <KWindowSystem>
 
 using namespace KTextEditor;
 
@@ -38,7 +40,7 @@ QWidget *findViewInternal(KTextEditor::View *view)
 class NoteProvider : public InlineNoteProvider
 {
 public:
-    QVector<int> inlineNotes(int line) const override
+    QList<int> inlineNotes(int line) const override
     {
         if (line == 0) {
             return {5, 10};
@@ -59,7 +61,7 @@ public:
         return QSize();
     }
 
-    void paintInlineNote(const InlineNote &note, QPainter &painter) const override
+    void paintInlineNote(const InlineNote &note, QPainter &painter, Qt::LayoutDirection) const override
     {
         if (note.position().column() == 5) {
             painter.setPen(Qt::darkGreen);
@@ -127,12 +129,11 @@ InlineNoteTest::~InlineNoteTest()
 void InlineNoteTest::testInlineNote()
 {
     KTextEditor::DocumentPrivate doc;
-    doc.setText(QLatin1String("xxxxxxxxxx\nxxxxxxxxxx"));
+    doc.setText(QStringLiteral("xxxxxxxxxx\nxxxxxxxxxx"));
 
     KTextEditor::ViewPrivate view(&doc, nullptr);
     view.show();
-
-    QTest::qWait(100);
+    view.resize(400, 300);
 
     view.setCursorPosition({0, 5});
     QCOMPARE(view.cursorPosition(), Cursor(0, 5));
@@ -145,16 +146,11 @@ void InlineNoteTest::testInlineNote()
 
     const auto xWidth = coordCol05.x() - coordCol04.x();
 
-    auto iface = qobject_cast<KTextEditor::InlineNoteInterface *>(&view);
-    QVERIFY(iface != nullptr);
-
     NoteProvider noteProvider;
-    const QVector<int> expectedColumns = {5, 10};
+    const QList<int> expectedColumns = {5, 10};
     QCOMPARE(noteProvider.inlineNotes(0), expectedColumns);
-    QCOMPARE(noteProvider.inlineNotes(1), QVector<int>());
-    iface->registerInlineNoteProvider(&noteProvider);
-
-    QTest::qWait(100);
+    QCOMPARE(noteProvider.inlineNotes(1), QList<int>());
+    view.registerInlineNoteProvider(&noteProvider);
 
     const auto newCoordCol04 = view.cursorToCoordinate({0, 4});
     const auto newCoordCol05 = view.cursorToCoordinate({0, 5});
@@ -174,10 +170,11 @@ void InlineNoteTest::testInlineNote()
     QCOMPARE(noteProvider.mouseMoveCount, 0);
     QVERIFY(noteProvider.lastUnderMouse == false);
 
-    // ATM fails on Windows, mark as such to be able to enforce test success in CI
-#ifdef Q_OS_WIN
-    QSKIP("Fails ATM, please fix");
-#endif
+    // mouse move only on X11
+    if (!KWindowSystem::isPlatformX11()) {
+        view.unregisterInlineNoteProvider(&noteProvider);
+        QSKIP("mouse moving only on X11");
+    }
 
     // move mouse onto first note
     auto internalView = findViewInternal(&view);
@@ -221,7 +218,7 @@ void InlineNoteTest::testInlineNote()
     QCOMPARE(noteProvider.noteActivatedCount, 1);
     QVERIFY(noteProvider.lastUnderMouse == false);
 
-    iface->unregisterInlineNoteProvider(&noteProvider);
+    view.unregisterInlineNoteProvider(&noteProvider);
 }
 
 // kate: indent-mode cstyle; indent-width 4; replace-tabs on;
