@@ -17,10 +17,13 @@
 #include <ktexteditor/message.h>
 #include <ktexteditor/movingcursor.h>
 
+#include <QScrollBar>
 #include <QTemporaryFile>
 #include <QtTestWidgets>
 
-#define testNewRow() (QTest::newRow(QString("line %1").arg(__LINE__).toLatin1().data()))
+#include <KWindowSystem>
+
+#define testNewRow() (QTest::newRow(QStringLiteral("line %1").arg(__LINE__).toLatin1().data()))
 
 using namespace KTextEditor;
 
@@ -39,7 +42,7 @@ KateViewTest::~KateViewTest()
 void KateViewTest::testCoordinatesToCursor()
 {
     KTextEditor::DocumentPrivate doc(false, false);
-    doc.setText("Hi World!\nHi\n");
+    doc.setText(QStringLiteral("Hi World!\nHi\n"));
 
     KTextEditor::View *view1 = static_cast<KTextEditor::View *>(doc.createView(nullptr));
     view1->resize(400, 300);
@@ -66,7 +69,7 @@ void KateViewTest::testCoordinatesToCursor()
     QCOMPARE(view1->coordinatesToCursor(view1->cursorPositionCoordinates()), KTextEditor::Cursor(2, 0));
 
     // same test again, but with message widget on top visible
-    KTextEditor::Message *message = new KTextEditor::Message("Jo World!", KTextEditor::Message::Information);
+    KTextEditor::Message *message = new KTextEditor::Message(QStringLiteral("Jo World!"), KTextEditor::Message::Information);
     doc.postMessage(message);
 
     // wait 500ms until show animation is finished, so the message widget is visible
@@ -82,7 +85,7 @@ void KateViewTest::testCoordinatesToCursor()
 void KateViewTest::testCursorToCoordinates()
 {
     KTextEditor::DocumentPrivate doc(false, false);
-    doc.setText("int a;");
+    doc.setText(QStringLiteral("int a;"));
 
     KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
     view->config()->setDynWordWrap(true);
@@ -96,10 +99,10 @@ void KateViewTest::testCursorToCoordinates()
 
 void KateViewTest::testReloadMultipleViews()
 {
-    QTemporaryFile file("XXXXXX.cpp");
+    QTemporaryFile file(QStringLiteral("XXXXXX.cpp"));
     file.open();
     QTextStream stream(&file);
-    const QString line = "const char* foo = \"asdf\"\n";
+    const QString line = QStringLiteral("const char* foo = \"asdf\"\n");
     for (int i = 0; i < 200; ++i) {
         stream << line;
     }
@@ -108,7 +111,7 @@ void KateViewTest::testReloadMultipleViews()
 
     KTextEditor::DocumentPrivate doc;
     QVERIFY(doc.openUrl(QUrl::fromLocalFile(file.fileName())));
-    QCOMPARE(doc.highlightingMode(), QString("C++"));
+    QCOMPARE(doc.highlightingMode(), QStringLiteral("C++"));
 
     KTextEditor::ViewPrivate *view1 = new KTextEditor::ViewPrivate(&doc, nullptr);
     KTextEditor::ViewPrivate *view2 = new KTextEditor::ViewPrivate(&doc, nullptr);
@@ -122,7 +125,7 @@ void KateViewTest::testReloadMultipleViews()
 void KateViewTest::testTabCursorOnReload()
 {
     // testcase for https://bugs.kde.org/show_bug.cgi?id=258480
-    QTemporaryFile file("XXXXXX.cpp");
+    QTemporaryFile file(QStringLiteral("XXXXXX.cpp"));
     file.open();
     QTextStream stream(&file);
     stream << "\tfoo\n";
@@ -143,14 +146,14 @@ void KateViewTest::testLowerCaseBlockSelection()
 {
     // testcase for https://bugs.kde.org/show_bug.cgi?id=258480
     KTextEditor::DocumentPrivate doc;
-    doc.setText("nY\nnYY\n");
+    doc.setText(QStringLiteral("nY\nnYY\n"));
 
     KTextEditor::ViewPrivate *view1 = new KTextEditor::ViewPrivate(&doc, nullptr);
     view1->setBlockSelection(true);
     view1->setSelection(Range(0, 1, 1, 3));
     view1->lowercase();
 
-    QCOMPARE(doc.text(), QString("ny\nnyy\n"));
+    QCOMPARE(doc.text(), QStringLiteral("ny\nnyy\n"));
 }
 
 namespace
@@ -177,7 +180,7 @@ void KateViewTest::testSelection()
     // => expected: selection from B to C
     // => actual: selection from A to C
 
-    QTemporaryFile file("XXXXXX.txt");
+    QTemporaryFile file(QStringLiteral("XXXXXX.txt"));
     file.open();
     QTextStream stream(&file);
     stream << "A\n"
@@ -193,7 +196,7 @@ void KateViewTest::testSelection()
     view->resize(100, 200);
     view->show();
 
-    QObject *internalView = findViewInternal(view);
+    auto *internalView = findViewInternal(view);
     QVERIFY(internalView);
 
     const QPoint afterA = view->cursorToCoordinate(Cursor(0, 1));
@@ -201,39 +204,49 @@ void KateViewTest::testSelection()
     const QPoint afterC = view->cursorToCoordinate(Cursor(2, 1));
 
     // click after A
-    auto me = QMouseEvent(QEvent::MouseButtonPress, afterA, Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    auto me = QMouseEvent(QEvent::MouseButtonPress, afterA, internalView->mapToGlobal(afterA), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
     QCoreApplication::sendEvent(internalView, &me);
 
-    auto me1 = QMouseEvent(QEvent::MouseButtonRelease, afterA, Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    auto me1 = QMouseEvent(QEvent::MouseButtonRelease, afterA, internalView->mapToGlobal(afterA), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
     QCoreApplication::sendEvent(internalView, &me1);
     QCOMPARE(view->cursorPosition(), Cursor(0, 1));
 
     // drag to right
-    auto me2 = QMouseEvent(QEvent::MouseButtonPress, afterA, Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    auto me2 = QMouseEvent(QEvent::MouseButtonPress, afterA, internalView->mapToGlobal(afterA), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
     QCoreApplication::sendEvent(internalView, &me2);
 
-    auto me3 = QMouseEvent(QEvent::MouseMove, afterA + QPoint(50, 0), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    auto me3 = QMouseEvent(QEvent::MouseMove,
+                           afterA + QPoint(50, 0),
+                           internalView->mapToGlobal(afterA + QPoint(50, 0)),
+                           Qt::LeftButton,
+                           Qt::LeftButton,
+                           Qt::NoModifier);
     QCoreApplication::sendEvent(internalView, &me3);
 
-    auto me4 = QMouseEvent(QEvent::MouseButtonRelease, afterA + QPoint(50, 0), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    auto me4 = QMouseEvent(QEvent::MouseButtonRelease,
+                           afterA + QPoint(50, 0),
+                           internalView->mapToGlobal(afterA + QPoint(50, 0)),
+                           Qt::LeftButton,
+                           Qt::LeftButton,
+                           Qt::NoModifier);
     QCoreApplication::sendEvent(internalView, &me4);
 
     QCOMPARE(view->cursorPosition(), Cursor(0, 1));
     QVERIFY(!view->selection());
 
     // click after C
-    auto me5 = QMouseEvent(QEvent::MouseButtonPress, afterC, Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    auto me5 = QMouseEvent(QEvent::MouseButtonPress, afterC, internalView->mapToGlobal(afterC), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
     QCoreApplication::sendEvent(internalView, &me5);
 
-    auto me6 = QMouseEvent(QEvent::MouseButtonRelease, afterC, Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    auto me6 = QMouseEvent(QEvent::MouseButtonRelease, afterC, internalView->mapToGlobal(afterC), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
     QCoreApplication::sendEvent(internalView, &me6);
 
     QCOMPARE(view->cursorPosition(), Cursor(2, 1));
     // shift+click after B
-    auto me7 = QMouseEvent(QEvent::MouseButtonPress, afterB, Qt::LeftButton, Qt::LeftButton, Qt::ShiftModifier);
+    auto me7 = QMouseEvent(QEvent::MouseButtonPress, afterB, internalView->mapToGlobal(afterB), Qt::LeftButton, Qt::LeftButton, Qt::ShiftModifier);
     QCoreApplication::sendEvent(internalView, &me7);
 
-    auto me8 = QMouseEvent(QEvent::MouseButtonRelease, afterB, Qt::LeftButton, Qt::LeftButton, Qt::ShiftModifier);
+    auto me8 = QMouseEvent(QEvent::MouseButtonRelease, afterB, internalView->mapToGlobal(afterB), Qt::LeftButton, Qt::LeftButton, Qt::ShiftModifier);
     QCoreApplication::sendEvent(internalView, &me8);
 
     QCOMPARE(view->cursorPosition(), Cursor(1, 1));
@@ -244,8 +257,8 @@ void KateViewTest::testDeselectByArrowKeys_data()
 {
     QTest::addColumn<QString>("text");
 
-    testNewRow() << "foobarhaz";
-    testNewRow() << "كلسشمن يتبكسب"; // We all win, translates Google
+    testNewRow() << QStringLiteral("foobarhaz");
+    testNewRow() << QStringLiteral("كلسشمن يتبكسب"); // We all win, translates Google
 }
 
 void KateViewTest::testDeselectByArrowKeys()
@@ -302,7 +315,7 @@ void KateViewTest::testDeselectByArrowKeys()
 void KateViewTest::testKillline()
 {
     KTextEditor::DocumentPrivate doc;
-    doc.insertLines(0, {"foo", "bar", "baz"});
+    doc.insertLines(0, {QStringLiteral("foo"), QStringLiteral("bar"), QStringLiteral("baz")});
 
     KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
 
@@ -314,7 +327,7 @@ void KateViewTest::testKillline()
     doc.clear();
     QVERIFY(doc.isEmpty());
 
-    doc.insertLines(0, {"foo", "bar", "baz", "xxx"});
+    doc.insertLines(0, {QStringLiteral("foo"), QStringLiteral("bar"), QStringLiteral("baz"), QStringLiteral("xxx")});
 
     view->setCursorPositionInternal(KTextEditor::Cursor(1, 2));
     view->shiftDown();
@@ -323,6 +336,39 @@ void KateViewTest::testKillline()
     QCOMPARE(doc.text(), QLatin1String("foo\nxxx\n"));
 }
 
+void KateViewTest::testKeyDeleteBlockSelection()
+{
+    KTextEditor::DocumentPrivate doc;
+    doc.insertLines(0, {QStringLiteral("foo"), QStringLiteral("12345"), QStringLiteral("bar"), QStringLiteral("baz")});
+
+    KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
+
+    view->setBlockSelection(true);
+    view->setSelection(Range(0, 1, 2, 1));
+    QCOMPARE(view->selectionRange(), Range(0, 1, 2, 1));
+
+    view->keyDelete();
+    QCOMPARE(doc.text(), QLatin1String("fo\n1345\nbr\nbaz\n"));
+    view->keyDelete();
+    QCOMPARE(doc.text(), QLatin1String("f\n145\nb\nbaz\n"));
+    view->keyDelete();
+    QCOMPARE(doc.text(), QLatin1String("f\n15\nb\nbaz\n"));
+    view->keyDelete();
+    QCOMPARE(doc.text(), QLatin1String("f\n1\nb\nbaz\n"));
+    view->keyDelete();
+    QCOMPARE(doc.text(), QLatin1String("f\n1\nb\nbaz\n"));
+
+    doc.clear();
+    QVERIFY(doc.isEmpty());
+
+    doc.insertLines(0, {QStringLiteral("foo"), QStringLiteral("12345"), QStringLiteral("bar"), QStringLiteral("baz")});
+
+    view->setSelection(Range(0, 1, 2, 3));
+
+    view->keyDelete();
+    QCOMPARE(doc.text(), QLatin1String("f\n145\nb\nbaz\n"));
+}
+
 void KateViewTest::testScrollPastEndOfDocument()
 {
     // bug 306745
@@ -360,7 +406,7 @@ void KateViewTest::testScrollPastEndOfDocument()
 
 void KateViewTest::testFoldFirstLine()
 {
-    QTemporaryFile file("XXXXXX.cpp");
+    QTemporaryFile file(QStringLiteral("XXXXXX.cpp"));
     file.open();
     QTextStream stream(&file);
     stream << "/**\n"
@@ -372,7 +418,7 @@ void KateViewTest::testFoldFirstLine()
 
     KTextEditor::DocumentPrivate doc;
     QVERIFY(doc.openUrl(QUrl::fromLocalFile(file.fileName())));
-    QCOMPARE(doc.highlightingMode(), QString("C++"));
+    QCOMPARE(doc.highlightingMode(), QStringLiteral("C++"));
 
     KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
     view->config()->setValue(KateViewConfig::FoldFirstLine, false);
@@ -404,18 +450,18 @@ void KateViewTest::testFoldFirstLine()
 // test for bug https://bugs.kde.org/374163
 void KateViewTest::testDragAndDrop()
 {
-    // ATM fails on Windows, mark as such to be able to enforce test success in CI
-#ifdef Q_OS_WIN
-    QSKIP("Fails ATM, please fix");
-#endif
+    // mouse move only on X11
+    if (!KWindowSystem::isPlatformX11()) {
+        QSKIP("mouse moving only on X11");
+    }
 
     KTextEditor::DocumentPrivate doc(false, false);
     doc.setText(
-        "line0\n"
-        "line1\n"
-        "line2\n"
-        "\n"
-        "line4");
+        QStringLiteral("line0\n"
+                       "line1\n"
+                       "line2\n"
+                       "\n"
+                       "line4"));
 
     KTextEditor::View *view = static_cast<KTextEditor::View *>(doc.createView(nullptr));
     view->show();
@@ -457,11 +503,11 @@ void KateViewTest::testDragAndDrop()
 
     // final tests of dragged text
     QCOMPARE(doc.text(),
-             QString("line0\n"
-                     "line2\n"
-                     "line1\n"
-                     "\n"
-                     "line4"));
+             QStringLiteral("line0\n"
+                            "line2\n"
+                            "line1\n"
+                            "\n"
+                            "line4"));
 
     QCOMPARE(view->cursorPosition(), KTextEditor::Cursor(3, 0));
     QCOMPARE(view->selectionRange(), Range(2, 0, 3, 0));
@@ -471,7 +517,7 @@ void KateViewTest::testDragAndDrop()
 void KateViewTest::testGotoMatchingBracket()
 {
     KTextEditor::DocumentPrivate doc(false, false);
-    doc.setText("foo(bar)baz[]");
+    doc.setText(QStringLiteral("foo(bar)baz[]"));
     //           0123456789
 
     KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
@@ -517,10 +563,10 @@ void KateViewTest::testFindSelected()
 {
     KTextEditor::DocumentPrivate doc(false, false);
     doc.setText(
-        "foo\n"
-        "bar\n"
-        "foo\n"
-        "bar\n");
+        QStringLiteral("foo\n"
+                       "bar\n"
+                       "foo\n"
+                       "bar\n"));
     //           0123456789
 
     KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
@@ -560,10 +606,10 @@ void KateViewTest::testTransposeWord()
 {
     KTextEditor::DocumentPrivate doc(false, false);
     doc.setText(
-        "swaps forward\n"
-        "wordAbove\n"
-        "wordLeft (_skips]Spaces.__And___}Sym_bols))))    wordRight)\n"
-        "wordBelow anotherWord yetAnotherWord\n");
+        QStringLiteral("swaps forward\n"
+                       "wordAbove\n"
+                       "wordLeft (_skips]Spaces.__And___}Sym_bols))))    wordRight)\n"
+                       "wordBelow anotherWord yetAnotherWord\n"));
 
     KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
     const KTextEditor::Cursor swaps(0, 2); // swa|ps
@@ -619,22 +665,22 @@ void KateViewTest::testTransposeWord()
 void KateViewTest::testFindMatchingFoldingMarker()
 {
     KTextEditor::DocumentPrivate doc(false, false);
-    doc.setMode(QString("Bash"));
-    doc.setHighlightingMode(QString("Bash"));
+    doc.setMode(QStringLiteral("Bash"));
+    doc.setHighlightingMode(QStringLiteral("Bash"));
 
     doc.setText(
-        "for i in 0 1 2; do\n"
-        "    if [[ i -lt 1 ]]; then echo $i; fi\n"
-        "    if [[ i -eq 2 ]]; then\n"
-        "        echo 'hello :)'\n"
-        "    fi\n"
-        "done\n");
+        QStringLiteral("for i in 0 1 2; do\n"
+                       "    if [[ i -lt 1 ]]; then echo $i; fi\n"
+                       "    if [[ i -eq 2 ]]; then\n"
+                       "        echo 'hello :)'\n"
+                       "    fi\n"
+                       "done\n"));
 
     KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
     KateViewInternal *viewInternal = view->getViewInternal();
 
-    const int ifvalue = doc.buffer().plainLine(1)->foldings()[0].foldingValue;
-    const int dovalue = doc.buffer().plainLine(0)->foldings()[0].foldingValue;
+    const auto ifvalue = doc.buffer().plainLine(1)->foldings()[0].foldingRegion;
+    const auto dovalue = doc.buffer().plainLine(0)->foldings()[0].foldingRegion;
 
     const KTextEditor::Range firstDo(0, 16, 0, 18);
     const KTextEditor::Range firstDoMatching(5, 0, 5, 4);
@@ -649,27 +695,27 @@ void KateViewTest::testFindMatchingFoldingMarker()
     QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDo.start(), dovalue, 2), KTextEditor::Range::invalid());
 
     // it must work from end folding to start folding too.
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDoMatching.start(), -dovalue, 2000), firstDo);
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDoMatching.start(), -dovalue, 2), KTextEditor::Range::invalid());
+    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDoMatching.start(), dovalue.sibling(), 2000), firstDo);
+    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstDoMatching.start(), dovalue.sibling(), 2), KTextEditor::Range::invalid());
 
     // folding in the same line
     QCOMPARE(viewInternal->findMatchingFoldingMarker(firstIf.start(), ifvalue, 2000), firstIfMatching);
-    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstIfMatching.start(), -ifvalue, 2000), firstIf);
+    QCOMPARE(viewInternal->findMatchingFoldingMarker(firstIfMatching.start(), ifvalue.sibling(), 2000), firstIf);
 }
 
 void KateViewTest::testUpdateFoldingMarkersHighlighting()
 {
     KTextEditor::DocumentPrivate doc(false, false);
-    doc.setMode(QString("Bash"));
-    doc.setHighlightingMode(QString("Bash"));
+    doc.setMode(QStringLiteral("Bash"));
+    doc.setHighlightingMode(QStringLiteral("Bash"));
 
     doc.setText(
-        "for i in 0 1 2; do\n"
-        "    if [[ i -lt 1 ]]; then echo $i; fi\n"
-        "    if [[ i -eq 2 ]]; then\n"
-        "        echo 'hello :)'\n"
-        "    fi\n"
-        "done\n");
+        QStringLiteral("for i in 0 1 2; do\n"
+                       "    if [[ i -lt 1 ]]; then echo $i; fi\n"
+                       "    if [[ i -eq 2 ]]; then\n"
+                       "        echo 'hello :)'\n"
+                       "    fi\n"
+                       "done\n"));
 
     KTextEditor::ViewPrivate *view = new KTextEditor::ViewPrivate(&doc, nullptr);
     KateViewInternal *viewInternal = view->getViewInternal();
