@@ -1,20 +1,8 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2010 Bernhard Beschow <bbeschow@cs.tu-berlin.de>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2010 Bernhard Beschow <bbeschow@cs.tu-berlin.de>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "plaintextsearch_test.h"
@@ -24,7 +12,7 @@
 #include <kateglobal.h>
 #include <kateplaintextsearch.h>
 
-#include <QtTestWidgets>
+#include <QTest>
 
 QTEST_MAIN(PlainTextSearchTest)
 
@@ -89,8 +77,8 @@ void PlainTextSearchTest::testSearchBackward()
     QFETCH(KTextEditor::Range, expectedResult);
 
     m_doc->setText(
-        QLatin1String("aaaa aaaa aaaa\n"
-                      "aaaa  aaaa"));
+        QStringLiteral("aaaa aaaa aaaa\n"
+                       "aaaa  aaaa"));
 
     QCOMPARE(m_search->search(QLatin1String("aaaa"), searchRange, true), expectedResult);
 }
@@ -123,7 +111,7 @@ void PlainTextSearchTest::testSingleLineDocument()
     QFETCH(KTextEditor::Range, forwardResult);
     QFETCH(KTextEditor::Range, backwardResult);
 
-    m_doc->setText(QLatin1String("a a a a a a a a a a a a"));
+    m_doc->setText(QStringLiteral("a a a a a a a a a a a a"));
 
     QCOMPARE(m_search->search(QLatin1String("a a a"), searchRange, false), forwardResult);
     QCOMPARE(m_search->search(QLatin1String("a a a"), searchRange, true), backwardResult);
@@ -161,9 +149,9 @@ void PlainTextSearchTest::testMultilineSearch()
     QFETCH(KTextEditor::Range, forwardResult);
 
     m_doc->setText(
-        QLatin1String("a a a\n"
-                      "a a\n"
-                      "a a a"));
+        QStringLiteral("a a a\n"
+                       "a a\n"
+                       "a a a"));
 
     QCOMPARE(m_search->search(pattern, inputRange, false), forwardResult);
 }
