@@ -15,7 +15,6 @@
 #include "katepartdebug.h"
 #include "kateview.h"
 
-
 #ifndef Q_OS_WIN
 #include <cerrno>
 #include <unistd.h>
@@ -29,6 +28,7 @@
 #include <QCryptographicHash>
 #include <QFile>
 #include <QFileInfo>
+#include <QStringEncoder>
 #include <QTemporaryFile>
 
 #if HAVE_KAUTH
@@ -47,13 +47,11 @@
 
 namespace Kate
 {
-TextBuffer::TextBuffer(KTextEditor::DocumentPrivate *parent, int blockSize, bool alwaysUseKAuth)
+TextBuffer::TextBuffer(KTextEditor::DocumentPrivate *parent, bool alwaysUseKAuth)
     : QObject(parent)
     , m_document(parent)
     , m_history(*this)
-    , m_blockSize(blockSize)
     , m_lines(0)
-    , m_lastUsedBlock(0)
     , m_revision(0)
     , m_editingTransactions(0)
     , m_editingLastRevision(0)
@@ -61,16 +59,11 @@ TextBuffer::TextBuffer(KTextEditor::DocumentPrivate *parent, int blockSize, bool
     , m_editingMinimalLineChanged(-1)
     , m_editingMaximalLineChanged(-1)
     , m_encodingProberType(KEncodingProber::Universal)
-    , m_fallbackTextCodec(nullptr)
-    , m_textCodec(nullptr)
     , m_generateByteOrderMark(false)
     , m_endOfLineMode(eolUnix)
     , m_lineLengthLimit(4096)
     , m_alwaysUseKAuthForSave(alwaysUseKAuth)
 {
-    // minimal block size must be > 0
-    Q_ASSERT(m_blockSize > 0);
-
     // create initial state
     clear();
 }
@@ -138,7 +131,6 @@ void TextBuffer::clear()
 
     // reset lines and last used block
     m_lines = 1;
-    m_lastUsedBlock = 0;
 
     // reset revision
     m_revision = 0;
@@ -165,16 +157,75 @@ TextLine TextBuffer::line(int line) const
     return m_blocks.at(blockIndex)->line(line);
 }
 
+int TextBuffer::cursorToOffset(KTextEditor::Cursor c) const
+{
+    if (!c.isValid() || c > document()->documentEnd()) {
+        return -1;
+    }
+
+    int off = 0;
+    int line = 0;
+    for (auto block : m_blocks) {
+        if (block->startLine() + block->lines() < c.line()) {
+            off += block->blockSize();
+            line += block->lines();
+        } else {
+            const int lines = block->lines();
+            for (int i = 0; i < lines; ++i) {
+                if (line >= c.line()) {
+                    off += qMin(c.column(), block->lineLength(line));
+                    return off;
+                }
+                off += block->lineLength(line) + 1;
+                line++;
+            }
+        }
+    }
+
+    Q_ASSERT(false);
+    return -1;
+}
+
+KTextEditor::Cursor TextBuffer::offsetToCursor(int offset) const
+{
+    if (offset >= 0) {
+        int off = 0;
+        for (auto block : m_blocks) {
+            if (off + block->blockSize() < offset) {
+                off += block->blockSize();
+            } else {
+                const int lines = block->lines();
+                int start = block->startLine();
+                int end = start + lines;
+                for (int line = start; line < end; ++line) {
+                    const int len = block->lineLength(line);
+                    if (off + len >= offset) {
+                        return KTextEditor::Cursor(line, offset - off);
+                    }
+                    off += len + 1;
+                }
+            }
+        }
+    }
+    return KTextEditor::Cursor::invalid();
+}
+
 QString TextBuffer::text() const
 {
     QString text;
+    qsizetype size = 0;
+    for (auto b : m_blocks) {
+        size += b->blockSize();
+    }
+    size -= 1; // remove -1, last newline
+    text.reserve(size);
 
     // combine all blocks
     for (TextBlock *block : std::as_const(m_blocks)) {
         block->text(text);
     }
 
-    // return generated string
+    Q_ASSERT(size == text.size());
     return text;
 }
 
@@ -195,10 +246,7 @@ bool TextBuffer::startEditing()
     m_editingMaximalLineChanged = -1;
 
     // transaction has started
-    Q_EMIT editingStarted();
-    if (m_document) {
-        Q_EMIT m_document->KTextEditor::Document::editingStarted(m_document);
-    }
+    Q_EMIT m_document->KTextEditor::Document::editingStarted(m_document);
 
     // first transaction started
     return true;
@@ -224,10 +272,7 @@ bool TextBuffer::finishEditing()
     Q_ASSERT(!editingChangedBuffer() || (m_editingMaximalLineChanged >= 0 && m_editingMaximalLineChanged < m_lines));
 
     // transaction has finished
-    Q_EMIT editingFinished();
-    if (m_document) {
-        Q_EMIT m_document->KTextEditor::Document::editingFinished(m_document);
-    }
+    Q_EMIT m_document->KTextEditor::Document::editingFinished(m_document);
 
     // last transaction finished
     return true;
@@ -269,10 +314,7 @@ void TextBuffer::wrapLine(const KTextEditor::Cursor position)
     balanceBlock(blockIndex);
 
     // emit signal about done change
-    Q_EMIT lineWrapped(position);
-    if (m_document) {
-        Q_EMIT m_document->KTextEditor::Document::lineWrapped(m_document, position);
-    }
+    Q_EMIT m_document->KTextEditor::Document::lineWrapped(m_document, position);
 }
 
 void TextBuffer::unwrapLine(int line)
@@ -322,10 +364,7 @@ void TextBuffer::unwrapLine(int line)
     balanceBlock(blockIndex);
 
     // emit signal about done change
-    Q_EMIT lineUnwrapped(line);
-    if (m_document) {
-        Q_EMIT m_document->KTextEditor::Document::lineUnwrapped(m_document, line);
-    }
+    Q_EMIT m_document->KTextEditor::Document::lineUnwrapped(m_document, line);
 }
 
 void TextBuffer::insertText(const KTextEditor::Cursor position, const QString &text)
@@ -360,10 +399,7 @@ void TextBuffer::insertText(const KTextEditor::Cursor position, const QString &t
     }
 
     // emit signal about done change
-    Q_EMIT textInserted(position, text);
-    if (m_document) {
-        Q_EMIT m_document->KTextEditor::Document::textInserted(m_document, position, text);
-    }
+    Q_EMIT m_document->KTextEditor::Document::textInserted(m_document, position, text);
 }
 
 void TextBuffer::removeText(KTextEditor::Range range)
@@ -406,10 +442,7 @@ void TextBuffer::removeText(KTextEditor::Range range)
     }
 
     // emit signal about done change
-    Q_EMIT textRemoved(range, text);
-    if (m_document) {
-        Q_EMIT m_document->KTextEditor::Document::textRemoved(m_document, range, text);
-    }
+    Q_EMIT m_document->KTextEditor::Document::textRemoved(m_document, range, text);
 }
 
 int TextBuffer::blockForLine(int line) const
@@ -419,53 +452,34 @@ int TextBuffer::blockForLine(int line) const
         qFatal("out of range line requested in text buffer (%d out of [0, %d])", line, lines());
     }
 
-    // we need blocks and last used block should not be negative
-    Q_ASSERT(!m_blocks.empty());
-    Q_ASSERT(m_lastUsedBlock >= 0);
+    size_t b = line / BufferBlockSize;
+    if (b >= m_blocks.size()) {
+        b = m_blocks.size() - 1;
+    }
 
-    // shortcut: try last block first
-    if (m_lastUsedBlock < (int)m_blocks.size()) {
-        // check if block matches
-        // if yes, just return again this block
-        TextBlock *block = m_blocks[m_lastUsedBlock];
-        const int start = block->startLine();
-        const int lines = block->lines();
-        if (start <= line && line < (start + lines)) {
-            return m_lastUsedBlock;
+    auto block = m_blocks[b];
+    if (block->startLine() <= line && line < block->startLine() + block->lines()) {
+        return b;
+    }
+
+    if (block->startLine() > line) {
+        for (int i = b - 1; i >= 0; --i) {
+            auto block = m_blocks[i];
+            if (block->startLine() <= line && line < block->startLine() + block->lines()) {
+                return i;
+            }
         }
     }
 
-    // search for right block
-    // use binary search
-    // if we leave this loop not by returning the found element we have an error
-    int blockStart = 0;
-    int blockEnd = m_blocks.size() - 1;
-    while (blockEnd >= blockStart) {
-        // get middle and ensure it is OK
-        int middle = blockStart + ((blockEnd - blockStart) / 2);
-        Q_ASSERT(middle >= 0);
-        Q_ASSERT(middle < (int)m_blocks.size());
-
-        // facts bout this block
-        TextBlock *block = m_blocks[middle];
-        const int start = block->startLine();
-        const int lines = block->lines();
-
-        // right block found, remember it and return it
-        if (start <= line && line < (start + lines)) {
-            m_lastUsedBlock = middle;
-            return middle;
-        }
-
-        // half our stuff ;)
-        if (line < start) {
-            blockEnd = middle - 1;
-        } else {
-            blockStart = middle + 1;
+    if (block->startLine() < line || (block->lines() == 0)) {
+        for (size_t i = b + 1; i < m_blocks.size(); ++i) {
+            auto block = m_blocks[i];
+            if (block->startLine() <= line && line < block->startLine() + block->lines()) {
+                return i;
+            }
         }
     }
 
-    // we should always find a block
     qFatal("line requested in text buffer (%d out of [0, %d[), no block found", line, lines());
     return -1;
 }
@@ -497,7 +511,7 @@ void TextBuffer::balanceBlock(int index)
     TextBlock *blockToBalance = m_blocks.at(index);
 
     // first case, too big one, split it
-    if (blockToBalance->lines() >= 2 * m_blockSize) {
+    if (blockToBalance->lines() >= 2 * BufferBlockSize) {
         // half the block
         int halfSize = blockToBalance->lines() / 2;
 
@@ -519,7 +533,7 @@ void TextBuffer::balanceBlock(int index)
     }
 
     // block still large enough, do nothing
-    if (2 * blockToBalance->lines() > m_blockSize) {
+    if (2 * blockToBalance->lines() > BufferBlockSize) {
         return;
     }
 
@@ -537,7 +551,7 @@ void TextBuffer::balanceBlock(int index)
 void TextBuffer::debugPrint(const QString &title) const
 {
     // print header with title
-    printf("%s (lines: %d bs: %d)\n", qPrintable(title), m_lines, m_blockSize);
+    printf("%s (lines: %d)\n", qPrintable(title), m_lines);
 
     // print all blocks
     for (size_t i = 0; i < m_blocks.size(); ++i) {
@@ -548,10 +562,10 @@ void TextBuffer::debugPrint(const QString &title) const
 bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLongLinesWrapped, int &longestLineLoaded, bool enforceTextCodec)
 {
     // fallback codec must exist
-    Q_ASSERT(m_fallbackTextCodec);
+    Q_ASSERT(!m_fallbackTextCodec.isEmpty());
 
     // codec must be set!
-    Q_ASSERT(m_textCodec);
+    Q_ASSERT(!m_textCodec.isEmpty());
 
     // first: clear buffer in any case!
     clear();
@@ -581,9 +595,9 @@ bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLo
         // in round 0 + 3 use the given encoding from user
         // in round 1 use 0, to trigger detection
         // in round 2 use fallback
-        QTextCodec *codec = m_textCodec;
+        QString codec = m_textCodec;
         if (i == 1) {
-            codec = nullptr;
+            codec.clear();
         } else if (i == 2) {
             codec = m_fallbackTextCodec;
         }
@@ -606,7 +620,7 @@ bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLo
 
             // bail out on encoding error, if not last round!
             if (encodingErrors && i < (enforceTextCodec ? 0 : 3)) {
-                BUFFER_DEBUG << "Failed try to load file" << filename << "with codec" << (file.textCodec() ? file.textCodec()->name() : "(null)");
+                BUFFER_DEBUG << "Failed try to load file" << filename << "with codec" << file.textCodec();
                 break;
             }
 
@@ -648,7 +662,7 @@ bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLo
                 unicodeData += lineLength;
 
                 // ensure blocks aren't too large
-                if (m_blocks.back()->lines() >= m_blockSize) {
+                if (m_blocks.back()->lines() >= BufferBlockSize) {
                     m_blocks.push_back(new TextBlock(this, m_blocks.back()->startLine() + m_blocks.back()->lines()));
                 }
 
@@ -686,7 +700,7 @@ bool TextBuffer::load(const QString &filename, bool &encodingErrors, bool &tooLo
     Q_ASSERT(m_lines > 0);
 
     // report CODEC + ERRORS
-    BUFFER_DEBUG << "Loaded file " << filename << "with codec" << m_textCodec->name() << (encodingErrors ? "with" : "without") << "encoding errors";
+    BUFFER_DEBUG << "Loaded file " << filename << "with codec" << m_textCodec << (encodingErrors ? "with" : "without") << "encoding errors";
 
     // report BOM
     BUFFER_DEBUG << (file.byteOrderMarkFound() ? "Found" : "Didn't find") << "byte order mark";
@@ -711,24 +725,30 @@ void TextBuffer::setDigest(const QByteArray &checksum)
     m_digest = checksum;
 }
 
-void TextBuffer::setTextCodec(QTextCodec *codec)
+void TextBuffer::setTextCodec(const QString &codec)
 {
     m_textCodec = codec;
 
     // enforce bom for some encodings
-    int mib = m_textCodec->mibEnum();
-    if (mib == 1013 || mib == 1014 || mib == 1015) { // utf16
-        setGenerateByteOrderMark(true);
-    }
-    if (mib == 1017 || mib == 1018 || mib == 1019) { // utf32
-        setGenerateByteOrderMark(true);
+    if (const auto setEncoding = QStringConverter::encodingForName(m_textCodec.toUtf8().constData())) {
+        for (const auto encoding : {QStringConverter::Utf16,
+                                    QStringConverter::Utf16BE,
+                                    QStringConverter::Utf16LE,
+                                    QStringConverter::Utf32,
+                                    QStringConverter::Utf32BE,
+                                    QStringConverter::Utf32LE}) {
+            if (setEncoding == encoding) {
+                setGenerateByteOrderMark(true);
+                break;
+            }
+        }
     }
 }
 
 bool TextBuffer::save(const QString &filename)
 {
     // codec must be set, else below we fail!
-    Q_ASSERT(m_textCodec);
+    Q_ASSERT(!m_textCodec.isEmpty());
 
     SaveResult saveRes = saveBufferUnprivileged(filename);
 
@@ -755,7 +775,7 @@ bool TextBuffer::save(const QString &filename)
 
 bool TextBuffer::saveBuffer(const QString &filename, KCompressionDevice &saveFile)
 {
-    std::unique_ptr<QTextEncoder> encoder(m_textCodec->makeEncoder(generateByteOrderMark() ? QTextCodec::DefaultConversion : QTextCodec::IgnoreHeader));
+    QStringEncoder encoder(m_textCodec.toUtf8().constData(), generateByteOrderMark() ? QStringConverter::Flag::WriteBom : QStringConverter::Flag::Default);
 
     // our loved eol string ;)
     QString eol = QStringLiteral("\n");
@@ -768,11 +788,11 @@ bool TextBuffer::saveBuffer(const QString &filename, KCompressionDevice &saveFil
     // just dump the lines out ;)
     for (int i = 0; i < m_lines; ++i) {
         // dump current line
-        saveFile.write(encoder->fromUnicode(line(i)->text()));
+        saveFile.write(encoder.encode(line(i)->text()));
 
         // append correct end of line string
         if ((i + 1) < m_lines) {
-            saveFile.write(encoder->fromUnicode(eol));
+            saveFile.write(encoder.encode(eol));
         }
 
         // early out on stream errors
@@ -878,7 +898,7 @@ bool TextBuffer::saveBufferEscalated(const QString &filename)
     qint64 read = -1;
     QCryptographicHash cryptographicHash(SecureTextBuffer::checksumAlgorithm);
     while ((read = temporaryBuffer->read(buffer, bufferLength)) > 0) {
-        cryptographicHash.addData(buffer, read);
+        cryptographicHash.addData(QByteArrayView(buffer, read));
         if (tempFile.write(buffer, read) == -1) {
             return false;
         }
@@ -945,7 +965,6 @@ void TextBuffer::markModifiedLinesAsSaved()
         block->markModifiedLinesAsSaved();
     }
 }
-
 }
 
 #include "moc_katetextbuffer.cpp"
