@@ -6,11 +6,10 @@
 #ifndef KATE_TEXTBUFFER_H
 #define KATE_TEXTBUFFER_H
 
+#include <QList>
 #include <QObject>
 #include <QSet>
 #include <QString>
-#include <QTextCodec>
-#include <QVector>
 
 #include "katetextblock.h"
 #include "katetexthistory.h"
@@ -33,6 +32,9 @@ class TextCursor;
 class TextBlock;
 class TextLineData;
 typedef std::shared_ptr<TextLineData> TextLine;
+
+constexpr int BufferBlockSize = 64;
+
 /**
  * Class representing a text buffer.
  * The interface is line based, internally the text will be stored in blocks of text lines.
@@ -55,10 +57,8 @@ public:
      * Construct an empty text buffer.
      * Empty means one empty line in one block.
      * @param parent parent qobject
-     * @param blockSize block size in lines the buffer should try to hold, default 64 lines
-     * @param alwaysUseKAuth only set this for unit testing purposes
      */
-    explicit TextBuffer(KTextEditor::DocumentPrivate *parent, int blockSize = 64, bool alwaysUseKAuth = false);
+    explicit TextBuffer(KTextEditor::DocumentPrivate *parent, bool alwaysUseKAuth = false);
 
     /**
      * Destruct the text buffer
@@ -93,9 +93,9 @@ public:
 
     /**
      * Set fallback codec for this buffer to use for load.
-     * @param codec fallback QTextCodec to use for encoding
+     * @param codec fallback to use for encoding
      */
-    void setFallbackTextCodec(QTextCodec *codec)
+    void setFallbackTextCodec(const QString &codec)
     {
         m_fallbackTextCodec = codec;
     }
@@ -104,7 +104,7 @@ public:
      * Get fallback codec for this buffer
      * @return currently in use fallback codec of this buffer
      */
-    QTextCodec *fallbackTextCodec() const
+    QString fallbackTextCodec() const
     {
         return m_fallbackTextCodec;
     }
@@ -113,15 +113,15 @@ public:
      * Set codec for this buffer to use for load/save.
      * Loading might overwrite this, if it encounters problems and finds a better codec.
      * Might change BOM setting.
-     * @param codec QTextCodec to use for encoding
+     * @param codec to use for encoding
      */
-    void setTextCodec(QTextCodec *codec);
+    void setTextCodec(const QString &codec);
 
     /**
      * Get codec for this buffer
      * @return currently in use codec of this buffer
      */
-    QTextCodec *textCodec() const
+    QString textCodec() const
     {
         return m_textCodec;
     }
@@ -238,6 +238,16 @@ public:
         return m_blocks.at(blockIndex)->lineLength(line);
     }
 
+    /**
+     * Retrieve offset in text for the given cursor position
+     */
+    int cursorToOffset(KTextEditor::Cursor c) const;
+
+    /**
+     * Retrieve cursor in text for the given offset
+     */
+    KTextEditor::Cursor offsetToCursor(int offset) const;
+
     /**
      * Retrieve text of complete buffer.
      * @return text for this buffer, lines separated by '\n'
@@ -383,42 +393,6 @@ Q_SIGNALS:
      */
     void saved(const QString &filename);
 
-    /**
-     * Editing transaction has started.
-     */
-    void editingStarted();
-
-    /**
-     * Editing transaction has finished.
-     */
-    void editingFinished();
-
-    /**
-     * A line got wrapped.
-     * @param position position where the wrap occurred
-     */
-    void lineWrapped(const KTextEditor::Cursor position);
-
-    /**
-     * A line got unwrapped.
-     * @param line line where the unwrap occurred
-     */
-    void lineUnwrapped(int line);
-
-    /**
-     * Text got inserted.
-     * @param position position where the insertion occurred
-     * @param text inserted text
-     */
-    void textInserted(const KTextEditor::Cursor position, const QString &text);
-
-    /**
-     * Text got removed.
-     * @param range range where the removal occurred
-     * @param text removed text
-     */
-    void textRemoved(KTextEditor::Range range, const QString &text);
-
 private:
     /**
      * Save result which indicates an abstract reason why the operation has
@@ -432,40 +406,35 @@ private:
      * @return index of found block
      */
     int blockForLine(int line) const;
+    // exported for movingrange_test
 
     /**
      * Fix start lines of all blocks after the given one
      * @param startBlock index of block from which we start to fix
      */
+    KTEXTEDITOR_NO_EXPORT
     void fixStartLines(int startBlock);
 
     /**
      * Balance the given block. Look if it is too small or too large.
      * @param index block to balance
      */
+    KTEXTEDITOR_NO_EXPORT
     void balanceBlock(int index);
 
-    /**
-     * Block for given index in block list.
-     * @param index block index
-     * @return block matching this index
-     */
-    TextBlock *blockForIndex(int index)
-    {
-        return m_blocks[index];
-    }
-
     /**
      * A range changed, notify the views, in case of attributes or feedback.
      * @param view which view is affected? nullptr for all views
      * @param lineRange line range that the change spans
      * @param needsRepaint do we need to trigger repaints? e.g. if ranges with attributes change
      */
+    KTEXTEDITOR_NO_EXPORT
     void notifyAboutRangeChange(KTextEditor::View *view, KTextEditor::LineRange lineRange, bool needsRepaint);
 
     /**
      * Mark all modified lines as lines saved on disk (modified line system).
      */
+    KTEXTEDITOR_NO_EXPORT
     void markModifiedLinesAsSaved();
 
     /**
@@ -474,18 +443,21 @@ private:
      * @param filename path name for display/debugging purposes
      * @param saveFile open device to write the buffer to
      */
+    KTEXTEDITOR_NO_EXPORT
     bool saveBuffer(const QString &filename, KCompressionDevice &saveFile);
 
     /**
      * Attempt to save the buffer content in the given filename location using
      * current privileges.
      */
+    KTEXTEDITOR_NO_EXPORT
     SaveResult saveBufferUnprivileged(const QString &filename);
 
     /**
      * Attempt to save the buffer content in the given filename location using
      * escalated privileges.
      */
+    KTEXTEDITOR_NO_EXPORT
     bool saveBufferEscalated(const QString &filename);
 
 public:
@@ -511,13 +483,20 @@ public:
      * @param rangesWithAttributeOnly only return ranges which have a attribute set
      * @return list of ranges affecting this line
      */
-    QVector<TextRange *> rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly) const
+    QList<TextRange *> rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly) const
     {
         // get block, this will assert on invalid line
         const int blockIndex = blockForLine(line);
         return m_blocks.at(blockIndex)->rangesForLine(line, view, rangesWithAttributeOnly);
     }
 
+    void rangesForLine(int line, KTextEditor::View *view, bool rangesWithAttributeOnly, QList<TextRange *> &outRanges) const
+    {
+        // get block, this will assert on invalid line
+        const int blockIndex = blockForLine(line);
+        return m_blocks.at(blockIndex)->rangesForLine(line, view, rangesWithAttributeOnly, outRanges);
+    }
+
     /**
      * Check if the given range pointer is still valid.
      * @return range pointer still belongs to range for this buffer
@@ -564,11 +543,6 @@ private:
      */
     TextHistory m_history;
 
-    /**
-     * block size in lines the buffer will try to hold
-     */
-    const int m_blockSize;
-
     /**
      * List of blocks which contain the lines of this buffer
      */
@@ -579,12 +553,6 @@ private:
      */
     int m_lines;
 
-    /**
-     * Last used block in the buffer. Is used for speeding up blockForLine.
-     * May contain invalid index, must be checked before using.
-     */
-    mutable int m_lastUsedBlock;
-
     /**
      * Revision of the buffer.
      */
@@ -634,12 +602,12 @@ private:
     /**
      * Fallback text codec to use
      */
-    QTextCodec *m_fallbackTextCodec;
+    QString m_fallbackTextCodec;
 
     /**
      * Text codec to use
      */
-    QTextCodec *m_textCodec;
+    QString m_textCodec;
 
     /**
      * Mime-Type used for transparent compression/decompression support
