@@ -7,14 +7,15 @@
 #ifndef KATE_TEXTLINE_H
 #define KATE_TEXTLINE_H
 
-#include <QSharedPointer>
+#include <KSyntaxHighlighting/foldingregion.h>
+#include <QList>
 #include <QString>
-#include <QVector>
-
-#include <ktexteditor_export.h>
 
+#include <KSyntaxHighlighting/FoldingRegion>
 #include <KSyntaxHighlighting/State>
 
+#include <memory>
+
 namespace Kate
 {
 /**
@@ -22,7 +23,7 @@ namespace Kate
  * For efficiency reasons, not only pure text is stored here, but also additional data.
  * Will be only accessed over shared pointers.
  */
-class KTEXTEDITOR_EXPORT TextLineData
+class TextLineData
 {
     /**
      * TexBlock is a friend class, only one allowed to touch the text content.
@@ -42,7 +43,7 @@ public:
          * @param _length length
          * @param _attributeValue attribute value
          */
-        explicit Attribute(int _offset = 0, int _length = 0, short _attributeValue = 0)
+        explicit Attribute(int _offset = 0, int _length = 0, int _attributeValue = 0)
             : offset(_offset)
             , length(_length)
             , attributeValue(_attributeValue)
@@ -62,7 +63,7 @@ public:
         /**
          * attribute value (to encode type of this range)
          */
-        short attributeValue;
+        int attributeValue;
     };
 
     /**
@@ -74,11 +75,13 @@ public:
         /**
          * Construct folding.
          * @param _offset offset of the folding start
-         * @param _foldingValue positive ones start foldings, negative ones end them
+         * @param _length length of the folding
+         * @param _foldingRegion folding region, includes if we start or end a region and the id
          */
-        Folding(int _offset, int _foldingValue)
+        Folding(int _offset, int _length, KSyntaxHighlighting::FoldingRegion _foldingRegion)
             : offset(_offset)
-            , foldingValue(_foldingValue)
+            , length(_length)
+            , foldingRegion(_foldingRegion)
         {
         }
 
@@ -88,31 +91,35 @@ public:
         int offset = 0;
 
         /**
-         * positive ones start foldings, negative ones end them
+         * length
          */
-        int foldingValue = 0;
+        int length = 0;
+
+        /**
+         * folding region, includes if we start or end a region and the id
+         */
+        KSyntaxHighlighting::FoldingRegion foldingRegion;
     };
 
     /**
      * Flags of TextLineData
      */
-    enum Flags { flagAutoWrapped = 1, flagFoldingStartAttribute = 2, flagFoldingStartIndentation = 4, flagLineModified = 8, flagLineSavedOnDisk = 16 };
+    enum Flags { flagAutoWrapped = 1, flagFoldingStartAttribute = 2, flagLineModified = 4, flagLineSavedOnDisk = 8 };
 
     /**
      * Construct an empty text line.
      */
-    TextLineData();
+    TextLineData() = default;
 
     /**
      * Construct an text line with given text.
      * @param text text to use for this line
      */
-    explicit TextLineData(const QString &text);
-
-    /**
-     * Destruct the text line
-     */
-    ~TextLineData();
+    explicit TextLineData(const QString &text)
+        : m_text(text)
+        , m_flags(0)
+    {
+    }
 
     /**
      * Accessor to the text contained in this line.
@@ -160,21 +167,7 @@ public:
     inline QChar at(int column) const
     {
         if (column >= 0 && column < m_text.length()) {
-            return m_text[column];
-        }
-
-        return QChar();
-    }
-
-    /**
-     * Same as at().
-     * @param column column you want char for
-     * @return char at given column or QChar()
-     */
-    inline QChar operator[](int column) const
-    {
-        if (column >= 0 && column < m_text.length()) {
-            return m_text[column];
+            return m_text.at(column);
         }
 
         return QChar();
@@ -210,21 +203,12 @@ public:
         return m_flags & flagLineSavedOnDisk;
     }
 
-    /**
-     * Is on this line a folding start?
-     * @return folding start line or not?
-     */
-    bool markedAsFoldingStart() const
-    {
-        return m_flags & (flagFoldingStartAttribute | flagFoldingStartIndentation);
-    }
-
     /**
      * Clear folding start status.
      */
     void clearMarkedAsFoldingStart()
     {
-        m_flags &= ~(flagFoldingStartAttribute | flagFoldingStartIndentation);
+        m_flags &= ~flagFoldingStartAttribute;
     }
 
     /**
@@ -236,33 +220,14 @@ public:
         return m_flags & flagFoldingStartAttribute;
     }
 
-    /**
-     * Is on this line a folding start per indentation?
-     * @return folding start line per indentation? or not?
-     */
-    bool markedAsFoldingStartIndentation() const
-    {
-        return m_flags & flagFoldingStartIndentation;
-    }
-
     /**
      * Mark as folding start line of an attribute based folding.
      */
     void markAsFoldingStartAttribute()
     {
-        clearMarkedAsFoldingStart();
         m_flags |= flagFoldingStartAttribute;
     }
 
-    /**
-     * Mark as folding start line of an indentation based folding.
-     */
-    void markAsFoldingStartIndentation()
-    {
-        clearMarkedAsFoldingStart();
-        m_flags |= flagFoldingStartIndentation;
-    }
-
     /**
      * Returns the line's length.
      */
@@ -281,15 +246,6 @@ public:
         return m_flags & flagAutoWrapped;
     }
 
-    /**
-     * Returns the complete text line (as a QString reference).
-     * @return text of this line, read-only
-     */
-    const QString &string() const
-    {
-        return m_text;
-    }
-
     /**
      * Returns the substring with \e length beginning at the given \e column.
      * @param column start column of text to return
@@ -387,7 +343,7 @@ public:
      * Accessor to attributes
      * @return attributes of this line
      */
-    const QVector<Attribute> &attributesList() const
+    const std::vector<Attribute> &attributesList() const
     {
         return m_attributesList;
     }
@@ -404,11 +360,12 @@ public:
     /**
      * Add new folding at end of foldings stored in this line
      * @param offset offset of folding start
-     * @param folding folding to add, positive to open, negative to close
+     * @param length length of the string that represents the folding
+     * @param foldingRegion folding region, includes if we start or end a region and the id
      */
-    void addFolding(int offset, int folding)
+    void addFolding(int offset, int length, KSyntaxHighlighting::FoldingRegion foldingRegion)
     {
-        m_foldings.emplace_back(offset, folding);
+        m_foldings.emplace_back(offset, length, foldingRegion);
     }
 
     /**
@@ -418,7 +375,7 @@ public:
      * @param pos position of attribute requested
      * @return value of attribute
      */
-    short attribute(int pos) const;
+    int attribute(int pos) const;
 
     /**
      * set auto-wrapped property
@@ -453,7 +410,7 @@ private:
     /**
      * attributes of this line
      */
-    QVector<Attribute> m_attributesList;
+    std::vector<Attribute> m_attributesList;
 
     /**
      * foldings of this line
@@ -474,8 +431,7 @@ private:
 /**
  * The normal world only accesses the text lines with shared pointers.
  */
-typedef QSharedPointer<TextLineData> TextLine;
-
+typedef std::shared_ptr<TextLineData> TextLine;
 }
 
 #endif
