@@ -55,7 +55,8 @@ public:
     }
 
 private:
-    typedef QMap<CodeCompletionModel::ExtraItemDataRoles, QVariant> RoleMap;
+    typedef std::pair<CodeCompletionModel::ExtraItemDataRoles, QVariant> RoleAndValue;
+    typedef std::vector<std::pair<CodeCompletionModel::ExtraItemDataRoles, QVariant>> RoleMap;
     RoleMap m_roleValues;
     QString m_customGroup;
     int m_groupSortingKey;
@@ -69,7 +70,7 @@ CodeCompletionModel *HierarchicalModelHandler::model() const
 
 bool HierarchicalModelHandler::hasHierarchicalRoles() const
 {
-    return !m_roleValues.isEmpty();
+    return !m_roleValues.empty();
 }
 
 void HierarchicalModelHandler::collectRoles(const QModelIndex &index)
@@ -99,7 +100,8 @@ void HierarchicalModelHandler::takeRole(const QModelIndex &index)
                 m_groupSortingKey = sortingKey.toInt();
             }
         } else {
-            m_roleValues[(CodeCompletionModel::ExtraItemDataRoles)v.toInt()] = value;
+            auto role = (CodeCompletionModel::ExtraItemDataRoles)v.toInt();
+            addValue(role, value);
         }
     } else {
         qCDebug(LOG_KTE) << "Did not return valid GroupRole in hierarchical completion-model";
@@ -108,9 +110,11 @@ void HierarchicalModelHandler::takeRole(const QModelIndex &index)
 
 QVariant HierarchicalModelHandler::getData(CodeCompletionModel::ExtraItemDataRoles role, const QModelIndex &index) const
 {
-    RoleMap::const_iterator it = m_roleValues.find(role);
+    auto it = std::find_if(m_roleValues.begin(), m_roleValues.end(), [role](const RoleAndValue &v) {
+        return v.first == role;
+    });
     if (it != m_roleValues.end()) {
-        return *it;
+        return it->second;
     } else {
         return index.data(role);
     }
@@ -124,7 +128,14 @@ HierarchicalModelHandler::HierarchicalModelHandler(CodeCompletionModel *model)
 
 void HierarchicalModelHandler::addValue(CodeCompletionModel::ExtraItemDataRoles role, const QVariant &value)
 {
-    m_roleValues[role] = value;
+    auto it = std::find_if(m_roleValues.begin(), m_roleValues.end(), [role](const RoleAndValue &v) {
+        return v.first == role;
+    });
+    if (it != m_roleValues.end()) {
+        it->second = value;
+    } else {
+        m_roleValues.push_back({role, value});
+    }
 }
 
 KateCompletionModel::KateCompletionModel(KateCompletionWidget *parent)
@@ -132,11 +143,8 @@ KateCompletionModel::KateCompletionModel(KateCompletionWidget *parent)
     , m_ungrouped(new Group({}, 0, this))
     , m_argumentHints(new Group(i18n("Argument-hints"), -1, this))
     , m_bestMatches(new Group(i18n("Best matches"), BestMatchesProperty, this))
+    , m_emptyGroups({m_ungrouped, m_argumentHints, m_bestMatches})
 {
-    m_emptyGroups.append(m_ungrouped);
-    m_emptyGroups.append(m_argumentHints);
-    m_emptyGroups.append(m_bestMatches);
-
     m_updateBestMatchesTimer = new QTimer(this);
     m_updateBestMatchesTimer->setSingleShot(true);
     connect(m_updateBestMatchesTimer, &QTimer::timeout, this, &KateCompletionModel::updateBestMatches);
@@ -145,12 +153,6 @@ KateCompletionModel::KateCompletionModel(KateCompletionWidget *parent)
     m_groupHash.insert(-1, m_argumentHints);
     m_groupHash.insert(BestMatchesProperty, m_argumentHints);
 
-    QList<QList<int>> mergedColumns;
-    mergedColumns << (QList<int>() << 0);
-    mergedColumns << (QList<int>() << 1 << 2 << 3 << 4);
-    mergedColumns << (QList<int>() << 5);
-    m_columnMerges = mergedColumns;
-
     createGroups();
 }
 
@@ -182,11 +184,11 @@ QVariant KateCompletionModel::data(const QModelIndex &index, int role) const
     if (!hasGroups() || groupOfParent(index)) {
         if (role == Qt::TextAlignmentRole) {
             int c = 0;
-            for (const QList<int> &list : std::as_const(m_columnMerges)) {
-                if (index.column() < c + list.size()) {
+            for (const auto &list : m_columnMerges) {
+                if (size_t(index.column()) < c + list.size()) {
                     c += list.size();
                     continue;
-                } else if (list.count() == 1 && list.first() == CodeCompletionModel::Scope) {
+                } else if (list.size() == 1 && list.front() == CodeCompletionModel::Scope) {
                     return Qt::AlignRight;
                 } else {
                     return QVariant();
@@ -211,7 +213,7 @@ QVariant KateCompletionModel::data(const QModelIndex &index, int role) const
             for (int column : m_columnMerges[index.column()]) {
                 QModelIndex sourceIndex = mapToSource(createIndex(index.row(), column, index.internalPointer()));
                 QVariant method = sourceIndex.data(CodeCompletionModel::HighlightingMethod);
-                if (method.type() == QVariant::Int && method.toInt() == CodeCompletionModel::CustomHighlighting) {
+                if (method.userType() == QMetaType::Int && method.toInt() == CodeCompletionModel::CustomHighlighting) {
                     return QVariant(CodeCompletionModel::CustomHighlighting);
                 }
             }
@@ -304,14 +306,14 @@ int KateCompletionModel::contextMatchQuality(const ModelRow &source) const
         QModelIndex hintIndex = row.second;
 
         QVariant depth = hintIndex.data(CodeCompletionModel::ArgumentHintDepth);
-        if (!depth.isValid() || depth.type() != QVariant::Int || depth.toInt() != 1) {
+        if (!depth.isValid() || depth.userType() != QMetaType::Int || depth.toInt() != 1) {
             continue; // Only match completion-items to argument-hints of depth 1(the ones the item will be given to as argument)
         }
 
         hintIndex.data(CodeCompletionModel::SetMatchContext);
 
         QVariant matchQuality = realIndex.data(CodeCompletionModel::MatchQuality);
-        if (matchQuality.isValid() && matchQuality.type() == QVariant::Int) {
+        if (matchQuality.isValid() && matchQuality.userType() == QMetaType::Int) {
             int m = matchQuality.toInt();
             if (m > bestMatch) {
                 bestMatch = m;
@@ -321,7 +323,7 @@ int KateCompletionModel::contextMatchQuality(const ModelRow &source) const
 
     if (m_argumentHints->filtered.empty()) {
         QVariant matchQuality = realIndex.data(CodeCompletionModel::MatchQuality);
-        if (matchQuality.isValid() && matchQuality.type() == QVariant::Int) {
+        if (matchQuality.isValid() && matchQuality.userType() == QMetaType::Int) {
             int m = matchQuality.toInt();
             if (m > bestMatch) {
                 bestMatch = m;
@@ -420,7 +422,7 @@ QModelIndex KateCompletionModel::index(int row, int column, const QModelIndex &p
         return createIndex(row, column, g);
     }
 
-    if (row >= m_rowTable.count()) {
+    if (size_t(row) >= m_rowTable.size()) {
         // qCWarning(LOG_KTE) << "Invalid index requested: row " << row << " beyond group range.";
         return QModelIndex();
     }
@@ -449,7 +451,7 @@ bool KateCompletionModel::hasIndex(int row, int column, const QModelIndex &paren
         return true;
     }
 
-    if (row >= m_rowTable.count()) {
+    if (size_t(row) >= m_rowTable.size()) {
         return false;
     }
 
@@ -471,12 +473,11 @@ QModelIndex KateCompletionModel::indexForGroup(Group *g) const
         return QModelIndex();
     }
 
-    int row = m_rowTable.indexOf(g);
-
-    if (row == -1) {
+    auto it = std::find(m_rowTable.begin(), m_rowTable.end(), g);
+    if (it == m_rowTable.end()) {
         return QModelIndex();
     }
-
+    int row = std::distance(m_rowTable.begin(), it);
     return createIndex(row, 0, quintptr(0));
 }
 
@@ -487,14 +488,19 @@ void KateCompletionModel::clearGroups()
     m_bestMatches->clear();
 
     // Don't bother trying to work out where it is
-    m_rowTable.removeAll(m_ungrouped);
-    m_emptyGroups.removeAll(m_ungrouped);
+    m_rowTable.erase(std::remove_if(m_rowTable.begin(),
+                                    m_rowTable.end(),
+                                    [this](Group *g) {
+                                        return (g == m_ungrouped) || (g == m_argumentHints) || (g == m_bestMatches);
+                                    }),
+                     m_rowTable.end());
 
-    m_rowTable.removeAll(m_argumentHints);
-    m_emptyGroups.removeAll(m_argumentHints);
-
-    m_rowTable.removeAll(m_bestMatches);
-    m_emptyGroups.removeAll(m_bestMatches);
+    m_emptyGroups.erase(std::remove_if(m_emptyGroups.begin(),
+                                       m_emptyGroups.end(),
+                                       [this](Group *g) {
+                                           return (g == m_ungrouped) || (g == m_argumentHints) || (g == m_bestMatches);
+                                       }),
+                        m_emptyGroups.end());
 
     qDeleteAll(m_rowTable);
     qDeleteAll(m_emptyGroups);
@@ -503,13 +509,10 @@ void KateCompletionModel::clearGroups()
     m_groupHash.clear();
     m_customGroupHash.clear();
 
-    m_emptyGroups.append(m_ungrouped);
+    m_emptyGroups.insert(m_emptyGroups.end(), {m_ungrouped, m_argumentHints, m_bestMatches});
+
     m_groupHash.insert(0, m_ungrouped);
-
-    m_emptyGroups.append(m_argumentHints);
     m_groupHash.insert(-1, m_argumentHints);
-
-    m_emptyGroups.append(m_bestMatches);
     m_groupHash.insert(BestMatchesProperty, m_bestMatches);
 }
 
@@ -574,7 +577,9 @@ void KateCompletionModel::createGroups()
         // no need to sort prefiltered, it is just the raw dump of everything
         // filtered is what gets displayed
         //         std::sort(g->prefilter.begin(), g->prefilter.end());
-        std::sort(g->filtered.begin(), g->filtered.end());
+        std::sort(g->filtered.begin(), g->filtered.end(), [this](const Item &l, const Item &r) {
+            return l.lessThan(this, r);
+        });
     }
 
     m_hasGroups = has_groups;
@@ -614,7 +619,7 @@ KateCompletionModel::Group *KateCompletionModel::createItem(const HierarchicalMo
             } else {
                 g = new Group(customGroup, 0, this);
                 g->customSortingKey = handler.customGroupingKey();
-                m_emptyGroups.append(g);
+                m_emptyGroups.push_back(g);
                 m_customGroupHash.insert(customGroup, g);
             }
         } else {
@@ -625,7 +630,7 @@ KateCompletionModel::Group *KateCompletionModel::createItem(const HierarchicalMo
     Item item = Item(g != m_argumentHints, this, handler, ModelRow(handler.model(), sourceIndex));
 
     if (g != m_argumentHints) {
-        item.match();
+        item.match(this);
     }
 
     g->addItem(item, notifyModel);
@@ -715,23 +720,12 @@ KateCompletionModel::Group *KateCompletionModel::fetchGroup(int attribute, bool
     Group *ret = new Group(title, attribute, this);
     ret->scope = QString();
 
-    m_emptyGroups.append(ret);
+    m_emptyGroups.push_back(ret);
     m_groupHash.insert(groupingAttribute, ret);
 
     return ret;
 }
 
-bool KateCompletionModel::hasGroups() const
-{
-    // qCDebug(LOG_KTE) << "m_groupHash.size()"<<m_groupHash.size();
-    // qCDebug(LOG_KTE) << "m_rowTable.count()"<<m_rowTable.count();
-    // We cannot decide whether there is groups easily. The problem: The code-model can
-    // be populated with a delay from within a background-thread.
-    // Proper solution: Ask all attached code-models(Through a new interface) whether they want to use grouping,
-    // and if at least one wants to, return true, else return false.
-    return m_hasGroups;
-}
-
 KateCompletionModel::Group *KateCompletionModel::groupForIndex(const QModelIndex &index) const
 {
     if (!index.isValid()) {
@@ -746,7 +740,7 @@ KateCompletionModel::Group *KateCompletionModel::groupForIndex(const QModelIndex
         return nullptr;
     }
 
-    if (index.row() < 0 || index.row() >= m_rowTable.count()) {
+    if (size_t(index.row()) >= m_rowTable.size()) {
         return m_ungrouped;
     }
 
@@ -765,13 +759,12 @@ QModelIndex KateCompletionModel::parent(const QModelIndex &index) const
             return QModelIndex();
         }
 
-        int row = m_rowTable.indexOf(g);
-
-        if (row == -1) {
+        auto it = std::find(m_rowTable.begin(), m_rowTable.end(), g);
+        if (it == m_rowTable.end()) {
             qCWarning(LOG_KTE) << "Couldn't find parent for index" << index;
             return QModelIndex();
         }
-
+        int row = std::distance(m_rowTable.begin(), it);
         return createIndex(row, 0, quintptr(0));
     }
 
@@ -783,7 +776,7 @@ int KateCompletionModel::rowCount(const QModelIndex &parent) const
     if (!parent.isValid()) {
         if (hasGroups()) {
             // qCDebug(LOG_KTE) << "Returning row count for toplevel " << m_rowTable.count();
-            return m_rowTable.count();
+            return m_rowTable.size();
         } else {
             // qCDebug(LOG_KTE) << "Returning ungrouped row count for toplevel " << m_ungrouped->filtered.count();
             return m_ungrouped->filtered.size();
@@ -813,7 +806,7 @@ QModelIndex KateCompletionModel::mapToSource(const QModelIndex &proxyIndex) cons
     }
 
     if (Group *g = groupOfParent(proxyIndex)) {
-        if (!m_rowTable.contains(g)) {
+        if (std::find(m_rowTable.begin(), m_rowTable.end(), g) == m_rowTable.end()) {
             qWarning() << Q_FUNC_INFO << "Stale proxy index for which there is no group";
             return {};
         }
@@ -888,12 +881,12 @@ QString KateCompletionModel::commonPrefixInternal(const QString &forcePrefix) co
 {
     QString commonPrefix; // isNull() = true
 
-    QList<Group *> groups = m_rowTable;
-    groups += m_ungrouped;
+    std::vector<Group *> groups = m_rowTable;
+    groups.push_back(m_ungrouped);
 
     for (Group *g : std::as_const(groups)) {
         for (const Item &item : std::as_const(g->filtered)) {
-            uint startPos = m_currentMatch[item.sourceRow().first].length();
+            uint startPos = currentCompletion(item.sourceRow().first).length();
             const QString candidate = item.name().mid(startPos);
 
             if (!candidate.startsWith(forcePrefix)) {
@@ -936,7 +929,7 @@ QString KateCompletionModel::commonPrefix(QModelIndex selectedIndex) const
         if (g && selectedIndex.row() < (int)g->filtered.size()) {
             // Follow the path of the selected item, finding the next non-empty common prefix
             Item item = g->filtered[selectedIndex.row()];
-            int matchLength = m_currentMatch[item.sourceRow().first].length();
+            int matchLength = currentCompletion(item.sourceRow().first).length();
             commonPrefix = commonPrefixInternal(item.name().mid(matchLength).left(1));
         }
     }
@@ -949,8 +942,8 @@ void KateCompletionModel::changeCompletions(Group *g)
     // This code determines what of the filtered items still fit
     // don't notify the model. The model is notified afterwards through a reset().
     g->filtered.clear();
-    std::remove_copy_if(g->prefilter.begin(), g->prefilter.end(), std::back_inserter(g->filtered), [](Item &item) {
-        return !item.match();
+    std::remove_copy_if(g->prefilter.begin(), g->prefilter.end(), std::back_inserter(g->filtered), [this](Item &item) {
+        return !item.match(this);
     });
 
     hideOrShowGroup(g, /*notifyModel=*/false);
@@ -1004,16 +997,18 @@ void KateCompletionModel::hideOrShowGroup(Group *g, bool notifyModel)
         if (g->filtered.empty()) {
             // Move to empty group list
             g->isEmpty = true;
-            int row = m_rowTable.indexOf(g);
-            if (row != -1) {
+            auto it = std::find(m_rowTable.begin(), m_rowTable.end(), g);
+
+            if (it != m_rowTable.end()) {
+                int row = std::distance(m_rowTable.begin(), it);
                 if (hasGroups() && notifyModel) {
                     beginRemoveRows(QModelIndex(), row, row);
                 }
-                m_rowTable.removeAt(row);
+                m_rowTable.erase(it);
                 if (hasGroups() && notifyModel) {
                     endRemoveRows();
                 }
-                m_emptyGroups.append(g);
+                m_emptyGroups.push_back(g);
             } else {
                 qCWarning(LOG_KTE) << "Group " << g << " not found in row table!!";
             }
@@ -1025,7 +1020,7 @@ void KateCompletionModel::hideOrShowGroup(Group *g, bool notifyModel)
             g->isEmpty = false;
 
             int row = 0; // Find row where to insert
-            for (int a = 0; a < m_rowTable.count(); a++) {
+            for (size_t a = 0; a < m_rowTable.size(); a++) {
                 if (g->orderBefore(m_rowTable[a])) {
                     row = a;
                     break;
@@ -1040,11 +1035,11 @@ void KateCompletionModel::hideOrShowGroup(Group *g, bool notifyModel)
                     beginInsertRows(QModelIndex(), 0, g->filtered.size());
                 }
             }
-            m_rowTable.insert(row, g);
+            m_rowTable.insert(m_rowTable.begin() + row, g);
             if (notifyModel) {
                 endInsertRows();
             }
-            m_emptyGroups.removeAll(g);
+            m_emptyGroups.erase(std::remove(m_emptyGroups.begin(), m_emptyGroups.end(), g), m_emptyGroups.end());
         }
     }
 }
@@ -1074,7 +1069,7 @@ void KateCompletionModel::debugStats()
     if (!hasGroups()) {
         qCDebug(LOG_KTE) << "Model groupless, " << m_ungrouped->filtered.size() << " items.";
     } else {
-        qCDebug(LOG_KTE) << "Model grouped (" << m_rowTable.count() << " groups):";
+        qCDebug(LOG_KTE) << "Model grouped (" << m_rowTable.size() << " groups):";
         for (Group *g : std::as_const(m_rowTable)) {
             qCDebug(LOG_KTE) << "Group" << g << "count" << g->filtered.size();
         }
@@ -1083,12 +1078,12 @@ void KateCompletionModel::debugStats()
 
 bool KateCompletionModel::hasCompletionModel() const
 {
-    return !m_completionModels.isEmpty();
+    return !m_completionModels.empty();
 }
 
 int KateCompletionModel::translateColumn(int sourceColumn) const
 {
-    if (m_columnMerges.isEmpty()) {
+    if (m_columnMerges.empty()) {
         return sourceColumn;
     }
 
@@ -1106,7 +1101,7 @@ int KateCompletionModel::translateColumn(int sourceColumn) const
     qCDebug(LOG_KTE) << k_funcinfo << columnMerge;*/
 
     int c = 0;
-    for (const QList<int> &list : m_columnMerges) {
+    for (const auto &list : m_columnMerges) {
         for (int column : list) {
             if (column == sourceColumn) {
                 return c;
@@ -1159,8 +1154,7 @@ int KateCompletionModel::countBits(int value)
 }
 
 KateCompletionModel::Item::Item(bool doInitialMatch, KateCompletionModel *m, const HierarchicalModelHandler &handler, ModelRow sr)
-    : model(m)
-    , m_sourceRow(sr)
+    : m_sourceRow(sr)
     , matchCompletion(StartsWithMatch)
     , m_haveExactMatch(false)
 {
@@ -1171,11 +1165,11 @@ KateCompletionModel::Item::Item(bool doInitialMatch, KateCompletionModel *m, con
     m_nameColumn = nameSibling.data(Qt::DisplayRole).toString();
 
     if (doInitialMatch) {
-        match();
+        match(m);
     }
 }
 
-bool KateCompletionModel::Item::operator<(const Item &rhs) const
+bool KateCompletionModel::Item::lessThan(KateCompletionModel *model, const Item &rhs) const
 {
     int ret = 0;
 
@@ -1200,8 +1194,8 @@ bool KateCompletionModel::Item::operator<(const Item &rhs) const
     ret = inheritanceDepth - rhs.inheritanceDepth;
 
     if (ret == 0) {
-        auto it = rhs.model->m_currentMatch.constFind(rhs.m_sourceRow.first);
-        if (it != rhs.model->m_currentMatch.cend()) {
+        auto it = model->m_currentMatch.constFind(rhs.m_sourceRow.first);
+        if (it != model->m_currentMatch.cend()) {
             const QString &filter = it.value();
             bool thisStartWithFilter = m_nameColumn.startsWith(filter, Qt::CaseSensitive);
             bool rhsStartsWithFilter = rhs.m_nameColumn.startsWith(filter, Qt::CaseSensitive);
@@ -1240,14 +1234,20 @@ void KateCompletionModel::Group::addItem(const Item &i, bool notifyModel)
     }
 
     if (notifyModel) {
-        prefilter.insert(std::upper_bound(prefilter.begin(), prefilter.end(), i), i);
+        auto comp = [this](const Item &left, const Item &right) {
+            return left.lessThan(model, right);
+        };
+        prefilter.insert(std::upper_bound(prefilter.begin(), prefilter.end(), i, comp), i);
     } else {
         prefilter.push_back(i);
     }
 
     if (i.isVisible()) {
         if (notifyModel) {
-            auto it = std::upper_bound(filtered.begin(), filtered.end(), i);
+            auto comp = [this](const Item &left, const Item &right) {
+                return left.lessThan(model, right);
+            };
+            auto it = std::upper_bound(filtered.begin(), filtered.end(), i, comp);
             const auto rowNumber = it - filtered.begin();
             model->beginInsertRows(groupIndex, rowNumber, rowNumber);
             filtered.insert(it, i);
@@ -1299,7 +1299,10 @@ KateCompletionModel::Group::Group(const QString &title, int attribute, KateCompl
 
 void KateCompletionModel::Group::resort()
 {
-    std::stable_sort(filtered.begin(), filtered.end());
+    auto comp = [this](const Item &left, const Item &right) {
+        return left.lessThan(model, right);
+    };
+    std::stable_sort(filtered.begin(), filtered.end(), comp);
     model->hideOrShowGroup(this);
 }
 
@@ -1317,11 +1320,6 @@ void KateCompletionModel::resort()
     updateBestMatches();
 }
 
-bool KateCompletionModel::Item::isValid() const
-{
-    return model && m_sourceRow.first && m_sourceRow.second.row() >= 0;
-}
-
 void KateCompletionModel::Group::clear()
 {
     prefilter.clear();
@@ -1442,7 +1440,7 @@ static inline bool containsAtWordBeginning(const QString &word, const QString &t
     return false;
 }
 
-KateCompletionModel::Item::MatchType KateCompletionModel::Item::match()
+KateCompletionModel::Item::MatchType KateCompletionModel::Item::match(KateCompletionModel *model)
 {
     const QString match = model->currentCompletion(m_sourceRow.first);
 
@@ -1477,12 +1475,6 @@ KateCompletionModel::Item::MatchType KateCompletionModel::Item::match()
         }
     }
 
-    if (matchCompletion == NoMatch) {
-        if (m_nameColumn.contains(match, Qt::CaseInsensitive)) {
-            matchCompletion = ContainsMatch;
-        }
-    }
-
     if (matchCompletion && match.length() == m_nameColumn.length()) {
         matchCompletion = PerfectMatch;
         m_haveExactMatch = true;
@@ -1512,7 +1504,7 @@ void KateCompletionModel::addCompletionModel(KTextEditor::CodeCompletionModel *m
         return;
     }
 
-    m_completionModels.append(model);
+    m_completionModels.push_back(model);
 
     connect(model, &KTextEditor::CodeCompletionModel::rowsInserted, this, &KateCompletionModel::slotRowsInserted);
     connect(model, &KTextEditor::CodeCompletionModel::rowsRemoved, this, &KateCompletionModel::slotRowsRemoved);
@@ -1583,7 +1575,7 @@ void KateCompletionModel::removeCompletionModel(CodeCompletionModel *model)
 void KateCompletionModel::makeGroupItemsUnique(bool onlyFiltered)
 {
     struct FilterItems {
-        FilterItems(KateCompletionModel &model, const QVector<KTextEditor::CodeCompletionModel *> &needShadowing)
+        FilterItems(KateCompletionModel &model, const QList<KTextEditor::CodeCompletionModel *> &needShadowing)
             : m_model(model)
             , m_needShadowing(needShadowing)
         {
@@ -1591,7 +1583,7 @@ void KateCompletionModel::makeGroupItemsUnique(bool onlyFiltered)
 
         QHash<QString, CodeCompletionModel *> had;
         KateCompletionModel &m_model;
-        const QVector<KTextEditor::CodeCompletionModel *> &m_needShadowing;
+        const QList<KTextEditor::CodeCompletionModel *> &m_needShadowing;
 
         void filter(std::vector<Item> &items)
         {
@@ -1631,7 +1623,7 @@ void KateCompletionModel::makeGroupItemsUnique(bool onlyFiltered)
         }
     };
 
-    QVector<KTextEditor::CodeCompletionModel *> needShadowing;
+    QList<KTextEditor::CodeCompletionModel *> needShadowing;
     for (KTextEditor::CodeCompletionModel *model : std::as_const(m_completionModels)) {
         KTextEditor::CodeCompletionModelControllerInterface *v4 = qobject_cast<KTextEditor::CodeCompletionModelControllerInterface *>(model);
         if (v4 && v4->shouldHideItemsWithEqualNames()) {
@@ -1675,7 +1667,7 @@ void KateCompletionModel::updateBestMatches()
 
             QVariant v = source.second.data(CodeCompletionModel::BestMatchesCount);
 
-            if (v.type() == QVariant::Int && v.toInt() > 0) {
+            if (v.userType() == QMetaType::Int && v.toInt() > 0) {
                 int quality = contextMatchQuality(source);
                 if (quality > 0) {
                     rowsForQuality.insert(quality, row);
@@ -1722,7 +1714,7 @@ void KateCompletionModel::updateBestMatches()
 
             QVariant v = source.second.data(CodeCompletionModel::BestMatchesCount);
 
-            if (v.type() == QVariant::Int && v.toInt() > 0) {
+            if (v.userType() == QMetaType::Int && v.toInt() > 0) {
                 // Return the best match with any of the argument-hints
 
                 int quality = contextMatchQuality(source);
@@ -1786,7 +1778,7 @@ void KateCompletionModel::rowSelected(const QModelIndex & /*row*/) const
 
 void KateCompletionModel::clearCompletionModels()
 {
-    if (m_completionModels.isEmpty()) {
+    if (m_completionModels.empty()) {
         return;
     }
 
