@@ -50,6 +50,7 @@
 #include <KActionCollection>
 #include <KCharsets>
 #include <KColorCombo>
+#include <KEncodingProber>
 #include <KFontRequester>
 #include <KMessageBox>
 #include <KProcess>
@@ -499,7 +500,7 @@ KateEditGeneralConfigTab::KateEditGeneralConfigTab(QWidget *parent)
     observeChanges(ui->chkStaticWordWrap);
     observeChanges(ui->cmbEncloseSelection);
     ui->lblBracketHelp->setFont(QFontDatabase::systemFont(QFontDatabase::SmallestReadableFont));
-    connect(ui->cmbEncloseSelection->lineEdit(), &QLineEdit::editingFinished, [=] {
+    connect(ui->cmbEncloseSelection->lineEdit(), &QLineEdit::editingFinished, [this] {
         const int index = ui->cmbEncloseSelection->currentIndex();
         const QString text = ui->cmbEncloseSelection->currentText();
         // Text removed? Remove item, but don't remove default data!
@@ -614,6 +615,7 @@ KateEditConfigTab::KateEditConfigTab(QWidget *parent)
     QVBoxLayout *layout = new QVBoxLayout(this);
     layout->setContentsMargins(0, 0, 0, 0);
     QTabWidget *tabWidget = new QTabWidget(this);
+    tabWidget->setDocumentMode(true);
 
     // add all tabs
     tabWidget->insertTab(0, editConfigTab, editConfigTab->name());
@@ -722,6 +724,7 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
 {
     QLayout *layout = new QVBoxLayout(this);
     QTabWidget *tabWidget = new QTabWidget(this);
+    tabWidget->setDocumentMode(true);
     layout->addWidget(tabWidget);
     layout->setContentsMargins(0, 0, 0, 0);
 
@@ -958,6 +961,7 @@ KateSaveConfigTab::KateSaveConfigTab(QWidget *parent)
     QVBoxLayout *layout = new QVBoxLayout(this);
     layout->setContentsMargins(0, 0, 0, 0);
     QTabWidget *tabWidget = new QTabWidget(this);
+    tabWidget->setDocumentMode(true);
 
     QWidget *tmpWidget = new QWidget(tabWidget);
     QVBoxLayout *internalLayout = new QVBoxLayout(tmpWidget);
@@ -1095,7 +1099,7 @@ void KateSaveConfigTab::apply()
     // set both standard and fallback encoding
     KateDocumentConfig::global()->setEncoding(KCharsets::charsets()->encodingForName(ui->cmbEncoding->currentText()));
 
-    KateGlobalConfig::global()->setProberType((KEncodingProber::ProberType)ui->cmbEncodingDetection->currentIndex());
+    KateGlobalConfig::global()->setValue(KateGlobalConfig::EncodingProberType, (KEncodingProber::ProberType)ui->cmbEncodingDetection->currentIndex());
     KateGlobalConfig::global()->setFallbackEncoding(KCharsets::charsets()->encodingForName(ui->cmbEncodingFallback->currentText()));
 
     KateDocumentConfig::global()->setEol(ui->cmbEOL->currentIndex());
@@ -1141,9 +1145,10 @@ void KateSaveConfigTab::reload()
     // encoding detection
     ui->cmbEncodingDetection->clear();
     bool found = false;
+    const auto proberType = (KEncodingProber::ProberType)KateGlobalConfig::global()->value(KateGlobalConfig::EncodingProberType).toInt();
     for (int i = 0; !KEncodingProber::nameForProberType((KEncodingProber::ProberType)i).isEmpty(); ++i) {
         ui->cmbEncodingDetection->addItem(KEncodingProber::nameForProberType((KEncodingProber::ProberType)i));
-        if (i == KateGlobalConfig::global()->proberType()) {
+        if (i == proberType) {
             ui->cmbEncodingDetection->setCurrentIndex(ui->cmbEncodingDetection->count() - 1);
             found = true;
         }
