@@ -7,8 +7,6 @@
 #ifndef KATE_MESSAGE_WIDGET_H
 #define KATE_MESSAGE_WIDGET_H
 
-#include "kateview.h"
-
 #include <QHash>
 #include <QPointer>
 #include <QWidget>
@@ -18,6 +16,7 @@
 namespace KTextEditor
 {
 class Message;
+class ViewPrivate;
 }
 
 class KMessageWidget;
@@ -27,7 +26,7 @@ class KateAnimation;
  * This class implements a message widget based on KMessageWidget.
  * It is used to show messages through the KTextEditior::MessageInterface.
  */
-class KTEXTEDITOR_EXPORT KateMessageWidget : public QWidget
+class KateMessageWidget : public QWidget
 {
     Q_OBJECT
 
@@ -40,10 +39,10 @@ public:
     /**
      * Post a new incoming message. Show either directly, or queue
      */
-    void postMessage(KTextEditor::Message *message, QList<QSharedPointer<QAction>> actions);
+    void postMessage(KTextEditor::Message *message, QList<std::shared_ptr<QAction>> actions);
 
     // for unit test
-    QString text() const;
+    KTEXTEDITOR_EXPORT QString text() const;
 
 protected Q_SLOTS:
     /**
@@ -78,7 +77,7 @@ private:
     // pointer to current Message
     QPointer<KTextEditor::Message> m_currentMessage;
     // shared pointers to QActions as guard
-    QHash<KTextEditor::Message *, QList<QSharedPointer<QAction>>> m_messageHash;
+    QHash<KTextEditor::Message *, QList<std::shared_ptr<QAction>>> m_messageHash;
     // the message widget, showing the actual contents
     KMessageWidget *m_messageWidget;
     // the show / hide effect controller
