@@ -1,24 +1,10 @@
-/*  SPDX-License-Identifier: LGPL-2.0-or-later
+/*
+    SPDX-FileCopyrightText: 2008-2009 Erlend Hamberg <ehamberg@gmail.com>
+    SPDX-FileCopyrightText: 2009 Paul Gideon Dann <pdgiddie@gmail.com>
+    SPDX-FileCopyrightText: 2011 Svyatoslav Kuzmich <svatoslav1@gmail.com>
+    SPDX-FileCopyrightText: 2012-2013 Simon St James <kdedevel@etotheipiplusone.com>
 
-    Copyright (C) 2008 - 2009 Erlend Hamberg <ehamberg@gmail.com>
-    Copyright (C) 2009 Paul Gideon Dann <pdgiddie@gmail.com>
-    Copyright (C) 2011 Svyatoslav Kuzmich <svatoslav1@gmail.com>
-    Copyright (C) 2012 - 2013 Simon St James <kdedevel@etotheipiplusone.com>
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KATEVI_VISUAL_VI_MODE_H
@@ -33,11 +19,8 @@ class InputModeManager;
 
 class VisualViMode : public NormalViMode
 {
-    Q_OBJECT
-
 public:
     explicit VisualViMode(InputModeManager *viInputModeManager, KTextEditor::ViewPrivate *view, KateViewInternal *viewInternal);
-    ~VisualViMode() override;
 
     void init();
 
@@ -56,7 +39,7 @@ public:
     void setVisualModeType(const ViMode mode);
     void saveRangeMarks();
 
-    void setStart(const KTextEditor::Cursor &c)
+    void setStart(const KTextEditor::Cursor c)
     {
         m_start = c;
     }
@@ -66,26 +49,26 @@ public:
         return m_start;
     }
 
-    void goToPos(const KTextEditor::Cursor &c);
+    void goToPos(const KTextEditor::Cursor c);
 
     ViMode getLastVisualMode() const
     {
         return m_lastVisualMode;
     }
 
-    const KTextEditor::Cursor &getStart() const
+    const KTextEditor::Cursor getStart() const
     {
         return m_start;
     }
 
     // Selects all lines in range;
-    void selectLines(const KTextEditor::Range &range);
+    void selectLines(KTextEditor::Range range);
 
     // Selects range between c1 and c2, but includes the end cursor position.
-    void selectInclusive(const KTextEditor::Cursor &c1, const KTextEditor::Cursor &c2);
+    void selectInclusive(const KTextEditor::Cursor c1, const KTextEditor::Cursor c2);
 
     // Select block between c1 and c2.
-    void selectBlockInclusive(const KTextEditor::Cursor &c1, const KTextEditor::Cursor &c2);
+    void selectBlockInclusive(const KTextEditor::Cursor c1, const KTextEditor::Cursor c2);
 
 protected:
     /**
@@ -96,10 +79,19 @@ protected:
      */
     void goToPos(const Range &r) override;
 
-private:
-    void initializeCommands();
+    /**
+     * Return commands available for this mode.
+     * Overwritten in sub classes to replace them, must be a stable reference!
+     */
+    virtual const std::vector<Command> &commands() override;
 
-public Q_SLOTS:
+    /**
+     * Return motions available for this mode.
+     * Overwritten in sub classes to replace them, must be a stable reference!
+     */
+    virtual const std::vector<Motion> &motions() override;
+
+public:
     /**
      * Updates the visual mode's range to reflect a new cursor position. This
      * needs to be called if modifying the range from outside the vi mode, e.g.
@@ -111,7 +103,6 @@ private:
     KTextEditor::Cursor m_start;
     ViMode m_mode, m_lastVisualMode;
 };
-
 }
 
 #endif /* KATEVI_VISUAL_VI_MODE_H */
